package com.ozs.system.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ozs.common.core.domain.AjaxResult;
import com.ozs.common.core.domain.entity.SysDept;
import com.ozs.common.utils.SecurityUtils;
import com.ozs.system.domain.SysDeptOrgan;
import com.ozs.system.mapper.SysDeptMapper;
import com.ozs.system.mapper.SysDeptOrganMapper;
import com.ozs.system.service.SysDeptOrganService;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.stream.Collectors;


/**
 * 参数配部门的机关指导处 服务层实现
 *
 * @author buzy
 */
@Service
public class SysDeptOrganServiceImpl implements SysDeptOrganService {

    @Autowired
    private SysDeptOrganMapper deptOrganMapper;
    @Autowired
    private SysDeptMapper sysDeptMapper;


    @Override
    public List<SysDeptOrgan> selectList(SysDeptOrgan deptOrgan) {
        LambdaQueryWrapper<SysDeptOrgan> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(SysDeptOrgan::getDeptId, deptOrgan.getDeptId());
        List<SysDeptOrgan> organs = deptOrganMapper.selectList(wrapper);
        return organs;
    }

    @Override
    public AjaxResult selectListMap() {
        HashMap<String, List<SysDeptOrgan>> hashMap = new LinkedHashMap<>();
        Long deptId = SecurityUtils.getDeptId();
        SysDept sysDept = sysDeptMapper.selectDeptById(deptId);
        //本部门
        LambdaQueryWrapper<SysDeptOrgan> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(SysDeptOrgan::getDeptId, deptId);
        List<SysDeptOrgan> organs = deptOrganMapper.selectList(wrapper);
        String[] ancestors = sysDept.getAncestors().split(",");
        //上级(除了顶级组织的每一级)
        if (ancestors.length > 2) {
            for (int i = 2; i < ancestors.length; i++) {
                LambdaQueryWrapper<SysDeptOrgan> wrapperT = new LambdaQueryWrapper<>();
                List<SysDeptOrgan> organsT;
                if (!ObjectUtils.isEmpty(ancestors[i])) {
                    String did = ancestors[i];
                    wrapperT.eq(SysDeptOrgan::getDeptId, did);
                    organsT = deptOrganMapper.selectList(wrapperT);
                    SysDept sysDeptTwo = sysDeptMapper.selectDeptById(Long.parseLong(did));
                    if (ObjectUtils.isEmpty(organsT)) {
                        hashMap.put(sysDeptTwo.getDeptName() + "(暂无数据)", organsT);
                    } else {
                        hashMap.put(sysDeptTwo.getDeptName(), organsT);
                    }
                }
            }
        }
        if (ObjectUtils.isEmpty(sysDept)) {
            hashMap.put(sysDept.getDeptName() + "(暂无数据)", organs);
        } else {
            hashMap.put(sysDept.getDeptName(), organs);
        }
        return AjaxResult.success(hashMap);
    }

    @Override
    public AjaxResult insertDeptOrgan(List<SysDeptOrgan> deptOrganList, Long userId) {
        if (!ObjectUtils.isEmpty(deptOrganList)) {
            LambdaQueryWrapper<SysDeptOrgan> wrapper = new LambdaQueryWrapper<>();
            wrapper.eq(SysDeptOrgan::getDeptId, deptOrganList.get(0).getDeptId());
            List<SysDeptOrgan> organs = deptOrganMapper.selectList(wrapper);
            List<Long> idList = organs.stream().map(SysDeptOrgan::getId).collect(Collectors.toList());
            List<Long> didList = deptOrganList.stream().map(SysDeptOrgan::getId).collect(Collectors.toList());
            for (Long aLong : idList) {
                if (!didList.contains(aLong)) {
                    deptOrganMapper.deleteById(aLong);
                }
            }
            if (!ObjectUtils.isEmpty(deptOrganList)) {
                for (SysDeptOrgan organ : deptOrganList) {
                    LambdaQueryWrapper<SysDeptOrgan> wrapperT = new LambdaQueryWrapper<>();
                    wrapperT.eq(SysDeptOrgan::getDeptId, deptOrganList.get(0).getDeptId());
                    List<SysDeptOrgan> organsT = deptOrganMapper.selectList(wrapperT);
                    List<String> organNameList = organsT.stream().map(SysDeptOrgan::getOrganName).collect(Collectors.toList());
                    if (!organNameList.contains(organ.getOrganName())) {
                        organ.setCreateBy(String.valueOf(userId));
                        organ.setCreateTime(new Date());
                        deptOrganMapper.insert(organ);
                    }
                }
            }
        }
        return AjaxResult.success();
    }

    @Override
    public AjaxResult deleteById(SysDeptOrgan deptOrgan) {
        if (deptOrganMapper.deleteById(deptOrgan.getId()) == 1) {
            return AjaxResult.success();
        } else {
            return AjaxResult.error("网络出小差，请稍后重试");
        }
    }

}
